	.286		;procesor conajmniej 286
        LOCALS	@@	;etykiety @@.... lokalne dla procedur
;
;	Interrupt Identification Register - IIR  (read only)
;	wg HelpPC 2.10 by DavidJurgens

;        76543210  2FA, 3FA Interrupt ID Register
;                 1 = no int. pending, 0=int. pending
;               Interrupt Id bits (see below)
;              16550  1 = timeout int. pending, 0 for 8250/16450
;            reserved (zero)
;          16550  set to 1 if FIFO queues are enabled

;        Bits
;         21       Meaning            Priority           To reset
;         00  modem-status-change      lowest      read MSR
;         01  transmit-register-empty  low         read IIR / write THR
;         10  data-available           high        read rec buffer reg
;         11  line-status              highest     read LSR

;        - interrupt pending flag uses reverse logic, 0 = pending, 1 = none
;        - interrupt will occur if any of the line status bits are set
;        - THRE bit is set when THRE register is emptied into the TSR

IRQBits		equ	06h		;maska bitw przerwa


;        Port 3F9 - Interrupt Enable Register - IER  (read/write)
;                   Baud Rate Divisor MSB if bit 7 of LCR is set  (read/write)
;
;        76543210  2F9, 3F9: Interrupt Enable Register
;                1 = enable data available int (and 16550 Timeout)
;               1 = enable THRE interrupt
;              1 = enable lines status interrupt
;             1 = enable modem-status-change interrupt
;         reserved (zero)
;
;       - 16550 will interrupt if data exists in the FIFO and isn't read
;         within the time it takes to receive four bytes or if no data is
;         received within the time it takes to receive four bytes.

IntReceiveEn		equ	00000001b	;maska odblokowujca przerwnie od odbirnika
IntTransmitEn		equ	00000010b	;od nadajnika
IntErrorEn		equ	00000100b	;od bdw
IntModemStatusEn	equ     00001000b       ;od zmiany linii


;------	MSR - Modem Status Register - tylko do odczytu---------------------
MSR_DCTS	equ	01H		; zmiana CTS
MSR_DDSR	equ	02H		; zmiana DSR
MSR_TERI	equ	04H		; TRAILING EDGE RING INDICATOR
MSR_DDCD	equ	08H		; zmiana DCD
;	zmiana bitw 0-3 MSR moe zglaszac przerwanie

;	stan lini CTS,DST,RI i DCD na starszych bitach
MSR_CTS 	equ	10H		; CTS aktywny
MSR_DSD 	equ	20H		; DATA SET READY
MSR_RI		equ	40H		; RING INDICATOR
MSR_DCD		equ	80H		; DCD aktywny

;------MCR - Modem Control Register ---------------------------------------
MCR_DTR 	equ	01H		; DATA TERMINAL READY
MCR_RTS 	equ	02H		; REQUEST TO SEND
MCR_OUT1	equ	04H		; OUT1
MCR_OUT2	equ	08H		; ten bit musi by ustawiony
					; aby dziaaly przerwania
MCR_LOOP	equ	10H		; Ptla testowa


;------ LSR - Line Status Register ----------------------------------------
;
LSR_DR		equ	01H		; DATA READY-odebrano znak
LSR_OR		equ	02H		; OVERRUN ERROR
LSR_PE		equ	04H		; PARITY ERROR
LSR_FE		equ	08H		; FRAMING ERROR
LSR_BI		equ	10H		; BREAK INTERRUPT
LSR_THRE	equ	20H		; TRANSMITTER HOLDING REGISTER
LSR_TSRE	equ	40H		; TX SHIFT REGISTER EMPTY

;------- Com_Status --------------------------------------------------------

in_buf_full	equ	0001h			;bufor przepeniony
in_or_err	equ	0002h			;Overrun error   = LSR_OR
in_pe_err	equ	0004h			;Parity error    = LSR_PE
in_fe_err	equ	0008h			;Framing error   = LSR_FE
in_bi_err	equ	0010h			;Break interrupt = LSR_BI
in_err_mask	equ	1eh			;maska bdw


out_buf_full	equ	0400h
MSR_changed	equ	0800h	; zmiana CTS,DSR,RI lub DCD;
CTS_asserted 	equ	1000H	; CTS aktywny
DSD_asserted 	equ	2000H	; DSR aktywny
RI_asserted	equ	4000H	; RI qktywny
DCD_asserted	equ	8000H	; DCD aktywny

;--------Flow_Status----------------------------------------------

xon_xoff_ctrl	equ	0001h
cts_rts_ctrl	equ	0002h	; kontrola RTS/CTS
rem_xoff	equ	0004h	; odebrano znak XOFF, nadawanie zablokowane
rem_xon		equ	0008h	; odebrano znak XON
send_xoff	equ	0010h	; naley wysa znak XOFF
send_xon	equ	0020h	; naley wysa znak XON
xon_sent	equ	0040h	; wysano xon
xoff_sent	equ	0080h	; wysano xoff

portOpened	equ	8000h	;port otwarty

;-------Try_Send----------------------------------
WRT_BLOCKED	equ	1	;Output blocked by XOFF or CTS requirement.
WRT_BUSY	equ	2	;Output not blocked but xmit buffer is busy.
WRT_NONE	equ	3	;Output possible but no character available.
WRT_DID 	equ	4	;Character sent.
WRT_OK          equ     5


DC1		equ     011h
DC3		equ	013h
XON     	equ	DC1
XOFF		equ	DC3


Master_PIC	equ	020h	;Master Interrupt controller port
Master_IMR	equ	021h	;Interrupt Mask register
Slave_PIC	equ	0a0h
eoi		equ	20h	;komenda 'end of interrupt'

;offsety rejestrow
com_data	equ	0
com_dl_lsb	equ	0	;mlodszy bajt podzielnika
com_dl_msb	equ	1	;starszy
IER_offs	equ	1	;INTERRUPT ENABLE REGISTER
IIR_offs	equ	2	;INTERRUPT IDENTIFICATION REG
LCR_offs	equ	3	;LINE CONTROL REG
MCR_offs	equ	4	;MODEM CONTROL REG offset
LSR_offs	equ	5	;LINE STATUS REG offset
MSR_offs	equ	6	;MODEM STATUS REG

trig_75		equ	256*75 / 100
trig_90		equ	256*90 /100

DATA	segment	word	public

_ComParam	struc
PortNr		dw	?
Baud		dw	?	;0=1200,1=2400,2=4800,3=9600,4=19200
Dial		dw	?
FlowCtrl	dw	?	;bit 0=1 ->Xon/Xoff, bit 1=1 -> CTS/RTS
DataBits	dw	?	;0=5,1=6,2=7,3=8
StopBits	dw	?	;0=1,1=2
Parity		dw	?	;0=none,1=odd,2=even
Auxiliary       dw      ?
Com_Status      dw      ?
_ComParam	ends

	extrn	ComPar:_ComParam        ;zdefinowane w ASYNC.PAS

IrqVectorNr		db	?	;Nr wektora przerwa
IrqDisableMask		db	?
IrqEnableMask		db	?
SaveIntVect		dd	?	;oryginalny wektor przerwa
PortAdr                 dw	?
IrqNr		        dw	?

Flow_Status	dw	?
in_error	dw	?
in_buffer_cnt	dw	?
In_tail		dw	?
In_head		dw	?
out_buffer_cnt	dw	?
Out_tail	dw	?
Out_head	dw	?

utracone_znaki	dw	1

in_buf_size		equ	256
in_buffer		DB	In_buf_size	dup	(?)
In_Buf_End		equ	offset $

out_buf_size		equ	256
out_buffer		DB	Out_Buf_size	dup	(?)
out_buf_end		equ	$
DATA	ends



CODE	segment	word public
	assume	cs:code
        assume	ds:data

IntReceive:	;Czytaj znak z UART'a i zapisz w buforze odbiorczym
		;jesli bufor zapelniony, ustaw flage bdu
		;
	mov	dx,PortAdr      	;dx=adres bazoway com'a
	in	al,dx			;czytaj znak
		;	Odczytany znak jest w AL. Jeli ustawiony jest tryb
                ;	softwer'owy XON/XOFF to nalezy sprawdzic czy odczytany
		;	to XOFF lub XON ...
	test    Flow_Status,xon_xoff_ctrl
	jz	read_1		;opuszczamy sprawdzanie XOFF/XON

	cmp	al,XON		;sprzwdzamy czy odebrano XON
	je	read_XON	;jesli tak skocz do kodu obslugi
                       			;tego zdarzenia
	cmp	al,XOFF 	;sprzawdzamy czy XOFF
	jne	read_1		;nie, skok dalej

		;	odebrano XOFF,  trzeba zaznaczy w statusie,
                ;	aby procedura nadawcza wstrzymaa nadawanie
		;	a do odebrani znaku XON
	or	Flow_Status,rem_xoff
	jmp	int_exit	;na tym koniec

read_XON:	;	odebrano XON
	and	Flow_Status,not rem_xoff 	;skasuj w statusie znacznik
	call	Try_Send			;wznw nadawanie
	sti					;
	jmp	int_exit			;koniec

read_1:		;	zapisz odebrany znak do bufora odbiorczego
                ;	zapisujemy zawsze na koncu kolejki
                ;	a pobieramy z pocztku
	mov	bx,In_tail	; pozycja ostatnio zapisanego znaku
	inc	bx		; przesu o jedn pozycj naprzd
	cmp	bx,in_buf_end	;czy wskaznik nie 'wyszed'
                                        ;poza koniec bufora
	jne	read_2		;ok wskaznik mniejszy ni koniec bufora
	mov	bx,offset in_buffer	;bedziemy zapisywa bufor ponownie
					;od poczatku
read_2:		;	sprawdzamy czy jest miejsce w buforze
	cmp	bx,In_head	;sprawdzamy czy nie zamazemy
				;danych jeszcze nie pobranych z bufora
	jne	read_3		;skok dalej,gdy jest miejsce w buforze

                ;	brak miejsca
       	inc	utracone_znaki		;zwiekszamy licznik utraconych
               				;znakw
	or	in_error,in_buf_full
	jmp	int_exit		;koniec

read_3:		; jest miejsce w buforze
		; jesli stosujemy softwarewy XON/XOFF lub harwarowe
		; sterowanie za pomoc RTS/DTR to nalezy sprawdzic
                ; czy nie nastepuje zapelnienie bufora
	test	Flow_Status,xon_xoff_ctrl	;czy ustawiona kontrola XOFF/XON
	jz	read_4			;nie, skocz dalej

	;	obowiazuje XON/XOFF
	mov	dx,in_buffer_cnt	;liczba znakow w buforze odbiorczym
	cmp	dx,trig_75		;czy przekracza arbitralnie
        				;przyjete 75% bufora
	jb	read_4			;tak , skok dalej

        ; zapelnienie bufora odbiorczego
	; osiagneo/przekroczyo 75%
	; naley wysla XOFF
	or	Flow_status,send_xoff	;zaznacz w statusie
               				;koniecznosc wysania XOFF
					;procedura wysylajaca
					;dane moze byc 'upiona',
                                        ;bo nie byo nic ostatnio
                                        ;do wysania, zatem nalezy
                                        ;j obudzi
	call	Try_Send		;co niniejszym czynimy
	sti

read_4:	;sprawdzamy kontrol RTS/CTS
	test	Flow_Status,cts_rts_ctrl
        jz	read_5			;brak kontroli CTS/RTS

	mov	dx,in_buffer_cnt	;liczba znakw w buforze odbiorczym
	cmp	dx,trig_90		;czy osiagnea 90% pojemnosci bufora?
	jne	read_5		;id dalej jeli nie

	;liczba znakw w buforze osiagnea 90% jego pojemnoi,
        ;zatem aby zablokowa dalszy napyw danych, ustawiamy RTS nieaktywne

	mov	cx,ax			;tymczasowo zachowujemy AX w CX
	mov	dx,PortAdr              ;adres portu
	add	dx,MCR_offs		;i rejestru MCR (modem control reg)
	in	al,dx			;aktualna warto MCR
	jmp	short $+2		;czas dla portu
	and	al,not (mcr_rts)	;zdejmujemy RTS
	out	dx,al
        				;ponownego uaktywnienia RTS powinna dokona
                                        ;procedura pobierajca znaki z bufora

	mov	ax,cx			;odtwarzamy AX
read_5:         ;wprowadz odczytany znak do bufora odbiorczego
		;bx wskazuje miejsce w buforze
	inc	in_buffer_cnt		;zaktualizuj licznik znakw
        mov	[bx],al			;zapisz znak
	mov	In_tail,bx		;zaktualizuj wskaznik bufora
;	or	in_status,in_buf_not_empty
	jmp	int_exit


czytaj_MSR	proc	near
;	czytaj Modem Status Register
;	na wjciu:	    AL = warto 4-rech MSB rejestru MSR

	mov	dx,PortAdr		;adres portu
	add	dx,MSR_offs		;offset rejestru MSR
	in	al,dx			;czytaj
					;odczyt kasuje przerwanie od MSR
	and	al,0f0h			;wyizoluj sygnaly CTS DTR RI DCD
	mov	dx,ComPar.com_status
	and	dh,not 0f0h
	or	dh,al			;dx-nowy status
	cmp	dx,ComPar.com_status
	jz      @@exit			;brak zmian
	or	dx,MSR_changed		;sygna dla aplikacji o zmianie
					;stanu lini
@@EXIT:	mov	ComPar.com_status,dx	;uaktualnij status
	ret
czytaj_MSR	endp


;==============================================================================
czytaj_LSR	proc	near	;czytaj Line Status Register

	push	dx 			;zachowaj DX
	mov	dx,PortAdr
	add	dx,LSR_offs		;adres LSR
	in	al,dx			;czytaj LSR i jednoczenie
					;kasuj przerwanie od LSR

	and	ax,11110b		;wyizoluj bedy:
               				;Overrun      - 0000 0010b
                                        ;Parity error - 0000 0100b
                                        ;Framing error- 0000 1000b
              				;Break        - 0001 0000b
	jz	czytaj_LSR_1		; dalej jeli brak bdw
        or	ComPar.Com_Status,ax	;zapisz bdy
	test	al,lsr_or		;czy OVERRUN
	jz	czytaj_LSR_1		;dalej jeli nie
	inc	utracone_znaki		;dodatkowo zwieksz licznik
					;utraconych znakw
czytaj_LSR_1:
	pop	dx
	ret
czytaj_LSR	endp


IntTransmit:	call	Try_Send
		sti
		jmp	int_exit


Try_Send	proc	near
;==============================================================================
;
;	Prba wysania nastpego znaku z bufora wyjciowego lub XON/XOFF
;
;	na wyjciu:
;
;		AL = WRT_BLOCKED if output blocked by XOFF or CTS requirement;
;		     WRT_BUSY	 if output not blocked but transmit buffer is
;				 busy;
;		     WRT_NONE	 if output possible but no character available;
;		     WRT_DID	 if character sent.
;		DX = undefined.
;		ES = undefined.
;		Interrupts are disabled.
;
;==============================================================================
		;
		;	sprawdzamy 'hardware control' (CTS)
		;
	test	Flow_Status,cts_rts_ctrl
	jz	@@00			;CTS nie jest niezbedny
	call	czytaj_MSR
	test	al,msr_cts		;czy CTS jest aktywny?
	jz	wysylanie_zablokowane	;Nie, nie mona wic nadawa

@@00:	;CTS jest aktywny lub nie jest niezbdny do nadawania
	test	Flow_Status,rem_xoff	;cz nadawanie zablokowane przez XOFF?
        jnz	wysylanie_zablokowane
;-------------------------------------------------------------------------
	;	czy Transmit Holding Register jest pusty?
	mov     dx,PortAdr
	add     dx,LSR_offs
;	cli
	in      al,dx
	test	al,lsr_thre
	jz	transmiter_busy 	;nie jest pusty,
					;zatem nie mona nadawa.
; mona wysa znak, ale czy rzeczywicie jest co do wysania?

	test	Flow_Status,send_xoff	;najpierw, czy trzeba wysa XOFF
	jz	@@01			;nie wysyamy XOFF.

	and	Flow_Status,not(send_xoff)
	or	Flow_Status,xoff_sent
	mov	al,XOFF 		;do wysania XOFF
	jmp	short @@04
@@01:
	test	Flow_Status,send_xon	;czy trzeba wysa XON
		jz	@@02		;nie wysyamy XON.

	and	Flow_Status,not send_xon;skasuj danie wysania XON
	or	Flow_Status,xon_sent	; i zaznacz, e wysano
	mov	al,XON			; do wysania XON
	jmp	short @@04
@@02:
	;czy jest znak w buforze nadawczym?
	mov	bx,Out_head
	cmp	bx,Out_tail
	je	write_next_none 	;Bufor pusty
	;wysyamy znak z bufora
	inc	bx			;point to next char
	cmp	bx,offset out_Buf_end 	;check for wrap-around
	jne	@@03			;skip unless wrap-around
	mov	bx,offset out_buffer	;point to beginning of buffer
@@03:   mov     al,[bx]  ;pobierz znak z poczatku kolejki
	mov     Out_head,bx
	dec	out_buffer_cnt		;zredukuj licznik znakw
	and	ComPar.com_status,not out_buf_full
@@04:
	mov	dx,PortAdr
	out	dx,al		;wylij zmak
	mov	al,WRT_DID
	jmp	short @@Exit

wysylanie_zablokowane:
	mov	al,WRT_BLOCKED
	jmp	short @@Exit

transmiter_busy:
	mov	al,WRT_BUSY
	jmp	short @@Exit

write_next_none:
	mov	al,WRT_NONE
@@Exit:	ret
Try_Send		endp


IntModemStatus	label	near
		; Status modemu si zmiani
	call	czytaj_MSR	; Jesli nastpia zmiana CTS
				; to ewentualnie wznawiamy nadawanie
	test	al,msr_cts	;czy aktywny CTS?
	jz	modem_1		;wyjd jesli nie
	call	Try_Send
	sti

modem_1:	; na zmiany innych syganow nie reagujemy
	jmp	int_exit

IntError	label	near	;moliwe powody przerwania:
	    			; overrun error
        	                ; parity error
                	        ; framing error
                        	; brak interrupt
	call	czytaj_LSR
	sti
	jmp	int_exit

com_int:
	push	ax
	push	bx
	push	cx
	push	dx
	push	ds
	mov	ax,seg DATA
	mov	ds,ax
	assume	ds:DATA
	mov	dx,PortAdr
	add	dx,IIR_offs
	in	al,dx		;czytaj interrupt id register
	sti
	test	al,1		;jesli najmodszy bit='0' to przerwanie
	jnz	int_eoi		;nia ma przerwania!
IntLoop:
        and	ax,IRQBits	;zamaskuj bity inne ni zwizane z przerwaniem
	mov	bx,ax
	jmp	word ptr cs:int_table[bx]

int_table	label	word
	DW	IntModemStatus
	DW	IntTransmit
	DW	IntReceive
	DW	IntError

int_exit:
	mov	dx,PortAdr		;sprawd czy nie pojawio si
	add	dx,IIR_offs		;nowe przerwanie
	in	al,dx
	test	al,1
	jz	IntLoop		;jest nowe przerwanie do obslugi
int_eoi:
	mov	al,EOI
	cli
;	out	Slave_Pic,al	;dla przerwa > 7
	out	Master_Pic,al	;wylij End of Interrupt
	pop	ds
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	iret

;function ComStatus:word;

		public	ComStatus
ComStatus	proc	far
	mov	ax,ComPar.com_status
        and	ComPar.com_status,not (MSR_changed+in_err_mask)
	retf
ComStatus	endp

;function OutCount:word;
		public	OutCount
OutCount	proc	far
	mov	ax,out_buffer_cnt
        retf
OutCount	endp

;function InCount:word;
		public	InCount
InCount	proc	far
	mov	ax,In_buffer_cnt
        retf
InCount	endp


; procedure ComWrite(b:byte);
		public	ComWrite
ComWrite	proc	FAR	;zakadamy e przed wywoaniem
				;sprawdzono czy jest miejsce w buforze
                                ; if ComStatus and out_buf_full <>0
				;  then ComWrite ....

	push	bp
        mov	bp,sp

	cli
	mov	bx,Out_tail
        cmp	bx,Out_head		;jesli Koniec<>Pocztek
        sti				;dyskusyjne
             				;bufor nie jest pusty i
	jnz	Add2Queue		;wstawiamy znak do bufora
        	;Bufor jest pusty i by moe nadajnik nie ma nic do
        	;wysania - jest bezczynny, zatem probujemy wysa znak
		;bezporednio
	call	Try_Send
	cmp	al,wrt_none
	jne	Add2Queue   		;wysanie znaku nie jest moliwe
					;zatem umieszczamy znak w buforze
					; wysyamy znak bezporednio
	mov	dx,PortAdr
	mov	al,[bp+06]
	out	dx,al
	mov     al,wrt_ok
	jmp	@@Exit

Add2Queue:
	inc	bx			;wkaznik na nastepn pozycj w buforze
	cmp	bx,offset Out_Buf_end	;czy wykroczylismy poza bufor
	jne	@@00
	mov	bx,offset Out_Buffer	;zapis od pocztku bufora
@@00:
	mov     ah,[bp+06]
	mov	[bx],ah			;umie znak w buforze
	mov     Out_tail,bx
	inc	out_buffer_cnt		;uaktualnij licznik znakw
	jmp     @@Leave
;                cmp	out_buffer_cnt,out_buf_size
;                jne     @@Exit
;		or	ComPar.com_status,out_buf_full
@@Exit:	sti
@@Leave:
        pop     bp
	retf	02
ComWrite	endp


SetRTS	proc	near
	mov	dx,PortAdr
	add	dx,MCR_offs		; adres Modem Control reg
	in	al,dx			;aktualny stan
	or	al,mcr_rts
        out	dx,al
        ret
SetRTS	endp

; function ComRead:char;
		public	ComRead
ComRead		proc	far
;	procedura pobiera znak z bufora odbiorczego
;	zakadamy, e przed jej wywoaniem sprawdzono
;	czy bufor nie jest pusty, za pomoc funcji GetInputStatus
;	cli
	mov	bx,In_head
	mov	ax,In_tail
;       sti
	cmp	ax,bx
	jne	@@00			;jest znak w buforze
	cli
	mov	In_Buffer_cnt,0		;na wszelki wypadek
	mov     al,0
	sti
	retf

@@00:	       ;czytamy znak z poczatku
	inc     bx
	cmp     bx,In_Buf_end
	jne     @@01
	mov     bx,offset In_Buffer
@@01: 	mov	al,[bx]
	mov     In_Head,bx
	dec	in_buffer_cnt		;zmiejsz licznik znakw
	jmp	@@10

	push	ax
	cmp	in_buffer_cnt,0		;czy bufor oprni si
	jne	@@Exit

	;bufor oprni si, zatem naley skontrolowa przepyw
	;(xon/xoff, RTS/CTS) i zaznaczy e nie ma wicej znakw
	and	ComPar.Com_Status,not (in_buf_full)

	test	Flow_Status,xoff_sent	;czy by wysany ostatni xoff
	jz	@@02			;jesli nie , dalej
	or	Flow_status,send_xon	;zaznacz,e naley wysa XON
	and	Flow_Status,not xoff_sent
	call	Try_Send
@@02:	test	Flow_Status,cts_rts_ctrl ;aktywna kontrola RTS/CTS
	jz	@@Exit			;jesli nie, dalej
	call	SetRTS
@@Exit: and	ComPar.Com_Status,not in_buf_full
	pop	ax			;pobierz odebrany znak ze stosu
@@10:	retf
ComRead		endp


;--------- Ustawianie parametrw portu: BaudRate, DataBits itd.--------
;        Line Control Register - LCR  (read/write)
;
;        76543210  2FB, 3FB  Line Control Register
;               word length select bits (see below)
;              0 = 1 stop bit, 1 = 1.5 or 2  (see note)
;             0 = no parity, 1 = parity (PEN)
;            0 = odd parity, 1 = even (EPS)
;           0 = parity disabled, 1 = enabled
;          0 = turn break off, 1 = force spacing break state
;         1 = baud rate divisor (DLAB); 0 = RBR, THR or IER

;        Bits
;         10     Word length bits
;         00 = 5 bits per character
;         01 = 6 bits per character
;         10 = 7 bits per character
;         11 = 8 bits per character

;        - stop bits = 1.5 for 5 bit words or 2 for 6, 7 or 8 bit words
;        - bit 7 changes the mode of registers 3F8 and 3F9.  If set these
;          registers become the LSB and MSB of the baud rate divisor.
;          Otherwise 3F8 is the Transmit/Receive Buffer Register and 3F9 is
;          the Interrupt Enable Register.


;==============================================================================
;	ComParam	= record
;	PortNr	: word;
;	PortAdr	: word;
;	IrqNr	: word;
;	Baud	: word;	{0=1200,1=2400,2=4800,3=9600,4=19200}
;	DataBits: word;	{0=5,1=6,2=7,3=8}
;       Parity	: word; {0=none,1=odd,2=even}
;	StopBits: word; {0=1,1=2}
;	end;
;==============================================================================

;	Baud Rate Divisors
;
;Baud_50		equ	900H
;Baud_75		equ	600H
;Baud_110		equ	417H
;Baud_134		equ	359H
;Baud_150		equ	300H
;Baud_300		equ	180H
Baud_600		equ	0C0H
Baud_1200	 	equ	060H
Baud_1800 		equ	040H
Baud_2000	 	equ	03AH
Baud_2400 		equ	030H
;Baud_3600 		equ	020H
Baud_4800 		equ	018H
Baud_7200 		equ	010H
Baud_9600 		equ	00CH
Baud_19200		equ	006H
Baud_38400		equ	003H
Baud_115200		equ	001H

DLAB_0		equ	00H		; SET DLAB OFF
DLAB_1		equ	80H		; SET DLAB ON

PARITY_NONE	equ	0
PARITY_ODD	equ	1
PARITY_EVEN	equ	2
;PARITY_MARK	equ	3
;PARITY_SPACE	equ	4

DATA_5		equ	0
DATA_6		equ	1
DATA_7		equ	2
DATA_8		equ	3

STOP_1		equ	0
STOP_2		equ	1

ENABLE_DLAB	equ	80H	; bit dostpu do DIVISOR LATCH

baud_table	label	word
	dw      Baud_600
	dw	Baud_1200
	dw	Baud_2400
	dw	Baud_4800
	dw	Baud_9600
	dw	Baud_19200
        dw	Baud_38400
	dw	Baud_115200

		public  OpenPort
OpenPort       	proc	far
	test	ComPar.Com_Status,PortOpened
	jz	@@Open
	jmp	@@Exit		;port jest ju otwarty
@@Open:

;	ustawienie pocztkowych wartoci zmiennych
	xor     ax,ax
	mov	ComPar.com_status,ax
	mov     In_head,offset In_Buffer
	mov     In_tail,offset In_Buffer
	mov     In_Buffer_cnt,ax

	mov     Out_head,offset Out_Buffer
	mov     Out_tail,offset Out_Buffer
	mov     Out_Buffer_Cnt,ax
	mov	ax,ComPar.FlowCtrl
	mov	Flow_Status,ax

;       ustawianie parametrw transmisji
        mov     bx,ComPar.PortNr
        cmp     bx,0
        jne     @@00
        mov     PortAdr,3f8h
        mov     IrqNr,4
        jmp     @@03

@@00:   cmp     bx,1
        jne     @@01
        mov     PortAdr,2f8h
        mov     IrqNr,3
        jmp     @@03
@@01:   cmp     bx,2
        jne     @@02
        mov     PortAdr,3e8h
        mov     IrqNr,4
        jmp     @@03
@@02:
        mov     PortAdr,2e8h
        mov     IrqNr,3
@@03:
	mov     dx,PortAdr
	mov	cx,dx			; zapamitaj w CX adres bazowy
	add	dx,LCR_offs	 	; adres LINE CONTROL REG.
	mov	al,Enable_Dlab		; Set dlab bit
	cli				; zablokuj przerwania
	out	dx,al			; od teraz pod adresem bazowym
					; znajduje si LSB latch divisor
	mov	dx,cx			; dx=adres bazowy
	mov	bx,ComPar.Baud
	shl	bx,1
	add	bx,offset cs:Baud_Table
	mov	al,cs:[bx]		; modszy bajt podzielnika
	out	dx,al			; zapisz do rejestru
	jmp	short $+2		; czas dla portu
	inc	dx			; adres MSB latch divisor
	mov	al,cs:[bx+1]		; starszy bajt podzielnika
	out	dx,al			; zapisz do rejestru
	jmp	short $+2		; czas dla portu

	; teraz do LCR zapisujemy pozostae parametry

	xor	ax,ax
	mov     al,byte ptr ComPar.DataBits
	mov	ah,byte ptr ComPar.StopBits
	shl	ah,2    		;przesun w lewo o 2 bity
	or	al,ah
	mov	ah,byte ptr ComPar.Parity
	shl	ah,3			;przesu w lewo o 3 bity
	or	al,ah
	mov	dx,PortAdr
	add	dx,LCR_offs
	out	dx,al
	sti

;	przerwanie
	mov	di,IrqNr
	mov	cx,di			; cl=IrqNr
	add	di,8			; przerwania harware'owe
					; przesunite o 8
	shl     di,2                    ; 4 bajty na wektor
	mov	al,1			; wyznaczenie maski przewa
	shl	al,cl			;
	mov	IrqDisableMask,al 	; maska blokujca przerwanie portu
	not	al			; zaneguj bity
	mov	IrqEnableMask,al	; maska odblokowujca przerwanie

	;zainstaluj procedure obsugi przerwania

	push    ds
	xor     bx,bx
	mov     ds,bx                   ;DS=0 , segment danych BIOSU
	mov     ax,offset com_int
	mov     bx,cs
	cli
	xchg	ax,[di] 		;zamie wektor z bx:ax
	xchg	bx,[di+2]
	sti
	pop     ds

	mov     word ptr SaveIntVect,ax    ;Zapamitaj oryginalny wektor
	mov     word ptr SaveIntVect+2,bx

;	odblokowanie przerwania w 8259
	mov	dx,Master_IMR		;adres INRERRUPT MASK REGISTER
	in	al,dx			;czytaj
	jmp	short $+2               ;czas dla portu
	and	al,IrqEnableMask	;wyzerowanie bitu uaktywnia przerwanie
	out	dx,al			;zapisz now mask

;	Oczy port

	mov	dx,PortAdr
	add     dx,LCR_offs		;adres LINE CONTROL REGISTER
	in	al,dx
	jmp	short $+2		;czas dla portu
	and	al,not Enable_Dlab 	;Wyzeruj na wszelki wypadek DLAB
	out	dx,al
	jmp	short $+2             	;czas dla portu
	mov     dx,PortAdr		;adres data reg
	in	al,dx		      	;oczy odbiornik
	jmp	short $+2             	;czas dla portu
	add     dx,MSR_offs		;
	in	al,dx			;czytaj MSR
	jmp	short $+2             	;czas dla portu


	mov	dx,PortAdr
	add     dx,LSR_offs
@@wait:
	in	al,dx
	jmp	short $+2       ;czas dla portu
	test	al,lsr_thre	;testuj transmit holding register.
	jz	@@wait		;czekaj a bdzie pusty
	cli
	mov     dx,PortAdr
	add     dx,IER_offs
;	mov	al,0		;zablokuj wszystkie zrda przerwa
;	out	dx,al
;	jmp	short $+2	; czas dla portu
	mov	al,IntReceiveEn+IntTransmitEn+IntErrorEn+IntModemStatusEn
	out	dx,al		; odblokuj przerwania
	jmp	short $+2	; czas dla portu

	mov	dx,PortAdr
	add	dx,MCR_offs
	in	al,dx
	or	al,MCR_OUT2+MCR_DTR
	test	Flow_Status,cts_rts_ctrl
	jz	@@100
	or	al,MCR_RTS
@@100:
	out	dx,al
	sti

;	Reset kontrolera przerwa 8259
	mov	al,eoi
	out	Master_PIC,al
	or	ComPar.Com_Status,PortOpened
        call	Czytaj_MSR
@@Exit:	retf
OpenPort       endp


	public	ClosePort
ClosePort	proc	far
	test	ComPar.Com_Status,PortOpened
        jz	@@Exit
        cli
	mov	dx,PortAdr
        add	dx,IER_offs
        xor	al,al
        out	dx,al	;zablokuj wszystkie przerwania UART'a

	; pozwalamy dokonczy ewantualn transmisj znaku
	mov	dx,PortAdr
        add	dx,LSR_offs
@@00:	in	al,dx
	and	al,LSR_THRE or LSR_TSRE
	cmp	al,LSR_THRE or LSR_TSRE
	jne	@@00
	mov	dx,PortAdr
	add	dx,MCR_offs
	xor	al,al
	out	dx,al

	; blokujemy przerwanie w kontrolerze 8259
	mov	dx,Master_IMR
	in	al,dx
	jmp	short $+2	; czas dla portu
	or	al,IrqDisableMask
	out	dx,al

	; przywracamy oryginalny wektor przerwa
	mov	ax,word ptr SaveIntVect
	mov	bx,word ptr SaveIntVect+2
	mov	di,IrqNr
	add     di,8
	shl     di,2
	push	ds
	xor     cx,cx
	mov     ds,cx                   ;DS=0
	cli
	mov     [di],ax
	mov     [di+2],bx
	sti
	pop     ds
	mov     ComPar.Com_Status,0
	sti
@@Exit:	retf
ClosePort	endp


CODE 	ENDS
	end
